/*

        lpr.c

        KipTOOLS - (C) Angel Ortega

*/

#include <stdio.h>
#include <time.h>

#include "kiptools.h"


/* buffer de entrada/salida */
char buffer[512];

/* nombre del trabajo */
char jobname[100];

/* nmero de secuencia */
int seqnum;

/* fichero de control */
char ctlfile[512];

/* hostname local */
char hostname[50];

/* puerto local (de 721 a 731) */
int local_port;


void exiterror(char * msg, int val)
{
        end_sockets();

        printf("%s: %s\n",msg, strerrorsocket());
        exit(val);
}


int main(int argc, char * argv[])
{
        int s;
        struct hostent * h;
        struct sockaddr_in server;
        struct sockaddr_in local;
        long l;
        int n,m;
        FILE * f;

        printf("Lpr - %s\n",KIPTOOLS_VERSION);
        printf("  (C) Angel Ortega 1995/1997.\n\n");

        if(argc<3)
        {
                printf("Usage: lpr {host} {printer name} {file[s]}\n\n");

                exit(1);
        }

        if(!init_sockets())
                exiterror("init",2);

        gethostname(hostname, sizeof(hostname)-1);

        /* transforma el IP */
        if((h=gethostbyname(argv[1]))==NULL)
        {
                printf("Error: host '%s' not found.\n", argv[1]);
                exiterror("gethostbyname",2);
        }

        memcpy((char *)&server.sin_addr,(char *)h->h_addr,h->h_length);
        server.sin_family=h->h_addrtype;
        server.sin_port=htons(515);

        seqnum=time(NULL);
        local_port=time(NULL);

        for(n=3;n<argc;n++)
        {
                if((f=fopen(argv[n],"rb"))==NULL)
                {
                        printf("Error: can't open '%s'\n", argv[n]);
                        exiterror("fopen",10);
                }

                /* pide un socket */
                if((s=socket(AF_INET, SOCK_STREAM, 0))==-1)
                        exiterror("socket",3);

                /* asigna alguno de los puertos permitidos */
                local_port++;
                local_port%=10;
                local.sin_port=htons(721+local_port);

                if(bind(s,(struct sockaddr *)&local,sizeof(local))==-1)
                        exiterror("bind",9);

                /* conecta */
                if(connect(s,(struct sockaddr *)&server,sizeof(server))==-1)
                        exiterror("connect",4);

                /* crea el nombre del trabajo */
                seqnum++;
                seqnum%=999;
                sprintf(jobname,"fA%03u%s", seqnum, hostname);

                printf("Printing '%s' as '%s'...\n",
                        argv[n], jobname);


                /* enva el comando de seleccin de impresora */

                printf("Setting printer...\t\t\t\r");

                sprintf(buffer, "\002%s\n", argv[2]);

                if(send(s,buffer,strlen(buffer),0)==-1)
                        exiterror("send",5);

                /* espera la respuesta */
                if(recv(s,buffer,sizeof(buffer)-1,0)==-1)
                        exiterror("recv",6);

                if(*buffer!=0)
                {
                        if(*buffer==1)
                                printf("\nPrinter '%s' error\t\t\t\n", argv[2]);
                        else
                                printf("\nError: %s\t\t\t\n", buffer);

                        fclose(f);
                        closesocket(s);

                        break;
                }


                /* enviando nombre de trabajo */
                printf("Setting data file...\t\t\t\r");

                /* calcula el tamao del fichero */
                fseek(f,0,SEEK_END);
                l=ftell(f);
                fseek(f,0,SEEK_SET);

                sprintf(buffer,"\003%ld d%s\n", l, jobname);

                if(send(s,buffer,strlen(buffer),0)==-1)
                        exiterror("send",5);

                /* espera la respuesta */
                if(recv(s,buffer,sizeof(buffer)-1,0)==-1)
                        exiterror("recv",6);

                if(*buffer!=0)
                {
                        if(*buffer==1)
                                printf("\nError: rejected job\t\t\t\n");
                        else
                        if(*buffer==2)
                                printf("\nError: No room for job\t\t\t\n");

                        fclose(f);
                        closesocket(s);

                        break;
                }


                /* vuelca el fichero */

                printf("Sending data file...\t\t\t\r");

                for(;;)
                {
                        if((m=fread(buffer,1,sizeof(buffer),f))==0)
                                break;

                        if(send(s,buffer,m,0)==-1)
                                exiterror("send",5);
                }

                fclose(f);

                /* confirma el envo */
                buffer[0]='\0';
                send(s,buffer,1,0);


                /* espera la respuesta */
                if(recv(s,buffer,sizeof(buffer)-1,0)==-1)
                        exiterror("recv",6);

                if(*buffer!=0)
                {
                        printf("\nError: Rejected file\t\t\t\n");

                        closesocket(s);

                        break;
                }


                /* envia el fichero de control */
                printf("Setting control file...\t\t\t\r");

                sprintf(ctlfile, "H%s\nP%s\nC%s\nL%s\n"
                "T%s\nld%s\nN%s\nUd%s\nJ%s\n",
                        argv[1],getlogin(),argv[1],getlogin(),argv[n],
                        jobname,argv[n],jobname,jobname);

                sprintf(buffer,"\2%d c%s\n", strlen(ctlfile), jobname);

                if(send(s,buffer,strlen(buffer),0)==-1)
                        exiterror("send",5);

                /* espera la respuesta */
                if(recv(s,buffer,sizeof(buffer)-1,0)==-1)
                        exiterror("recv",6);

                if(*buffer!=0)
                {
                        if(*buffer==1)
                                printf("\nError: Bad connection\t\t\t\n");
                        else
                        if(*buffer==2)
                                printf("\nError: No room for job\t\t\t\n");
                        else
                                printf("\nUnknown error\t\t\t\n");

                        closesocket(s);

                        break;
                }


                /* enva el fichero de control */
                printf("Sending control file...\t\t\t\r");

                if(send(s,ctlfile,strlen(ctlfile),0)==-1)
                        exiterror("send",5);

                /* confirma el envo */
                buffer[0]='\0';
                send(s,buffer,1,0);


                /* espera la respuesta */
                if(recv(s,buffer,sizeof(buffer)-1,0)==-1)
                        exiterror("recv",6);

                if(*buffer!=0)
                {
                        printf("\nError: Rejected job\t\t\t\n");

                        closesocket(s);

                        break;
                }


                printf("Accepted job.\t\t\t\n\n");

                /* cierra el socket */
                if(closesocket(s)==-1)
                        exiterror("close",7);
        }

        end_sockets();

        return(0);
}
